]> rtime.felk.cvut.cz Git - frescor/ffmpeg.git/commitdiff
w32threads by (Gildas Bazin <gbazin at altern dot org>)
authormichael <michael@9553f0bf-9b14-0410-a0b8-cfaf0461ba5b>
Thu, 19 Feb 2004 20:29:26 +0000 (20:29 +0000)
committermichael <michael@9553f0bf-9b14-0410-a0b8-cfaf0461ba5b>
Thu, 19 Feb 2004 20:29:26 +0000 (20:29 +0000)
git-svn-id: file:///var/local/repositories/ffmpeg/trunk@2803 9553f0bf-9b14-0410-a0b8-cfaf0461ba5b

configure
ffmpeg.c
libavcodec/Makefile
libavcodec/pthread.c
libavcodec/w32thread.c [new file with mode: 0644]

index 191ce3c2170bb400be39dd2820d093a31c271589..eb41868f51529b3785dea59b96504a69670debc3 100755 (executable)
--- a/configure
+++ b/configure
@@ -707,7 +707,7 @@ if (zlibVersion() != ZLIB_VERSION)
 return 0;
 }
 EOF
-$cc -o $TMPE $TMPC -lz 2> /dev/null || zlib="no"
+$cc $CFLAGS -o $TMPE $TMPC -lz 2> /dev/null || zlib="no"
 # $TMPE 2> /dev/null > /dev/null || zlib="no"
 # XXX: more tests needed - runtime test
 fi
@@ -1165,6 +1165,8 @@ fi
 if test "$mingw32" = "yes" ; then
   echo "#define CONFIG_WIN32 1" >> $TMPH
   echo "CONFIG_WIN32=yes" >> config.mak
+  echo "HAVE_W32THREADS=yes" >> config.mak
+  echo "#define HAVE_W32THREADS 1" >> $TMPH
   echo "#ifndef __MINGW32__" >> $TMPH
   echo "#define __MINGW32__ 1" >> $TMPH
   echo "#endif" >> $TMPH
index a8c3a93f6fb58283aad34a70d0a883e99b5c1e9d..dec1ca6a5e61db9980820c93f63a28c3ef388972 100644 (file)
--- a/ffmpeg.c
+++ b/ffmpeg.c
@@ -2036,7 +2036,7 @@ static void opt_sc_threshold(const char *arg)
 static void opt_thread_count(const char *arg)
 {
     thread_count= atoi(arg);
-#ifndef HAVE_PTHREADS
+#if !defined(HAVE_PTHREADS) && !defined(HAVE_W32THREADS)
     fprintf(stderr, "Warning: not compiled with thread support, using thread emulation\n");
 #endif
 }
@@ -2399,9 +2399,9 @@ static void opt_output_file(const char *filename)
                 exit(1);
             }
             avcodec_get_context_defaults(&st->codec);
-#ifdef HAVE_PTHREADS
+#if defined(HAVE_PTHREADS) || defined(HAVE_W32THREADS)
             if(thread_count>1)
-                avcodec_pthread_init(&st->codec, thread_count);
+                avcodec_thread_init(&st->codec, thread_count);
 #endif
 
             video_enc = &st->codec;
@@ -2590,9 +2590,9 @@ static void opt_output_file(const char *filename)
                 exit(1);
             }
             avcodec_get_context_defaults(&st->codec);
-#ifdef HAVE_PTHREADS
+#if defined(HAVE_PTHREADS) || defined(HAVE_W32THREADS)
             if(thread_count>1)
-                avcodec_pthread_init(&st->codec, thread_count);
+                avcodec_thread_init(&st->codec, thread_count);
 #endif
 
             audio_enc = &st->codec;
index f90976c0a4e43d374dc7f80753e4894fad18dc3d..a464610d6df9bb33125e7ff6374160e1786f4ce8 100644 (file)
@@ -38,6 +38,10 @@ ifeq ($(HAVE_PTHREADS),yes)
 OBJS+= pthread.o
 endif
 
+ifeq ($(HAVE_W32THREADS),yes)
+OBJS+= w32thread.o
+endif
+
 ifeq ($(AMR_WB),yes)
 OBJS+= amr.o amrwb_float/dec_acelp.o amrwb_float/dec_dtx.o amrwb_float/dec_gain.o \
                amrwb_float/dec_if.o amrwb_float/dec_lpc.o amrwb_float/dec_main.o \
index 6f98dc41f4e0965f0ae27c4f23187501fab8e823..770f9a54ddb4e242324dd5ecec8438e99703321d 100644 (file)
@@ -54,10 +54,10 @@ static void * thread_func(void *v){
 }
 
 /**
* free what has been allocated by avcodec_pthread_init().
* must be called after decoding has finished, especially dont call while avcodec_pthread_execute() is running
  */
-void avcodec_pthread_free(AVCodecContext *s){
+void avcodec_thread_free(AVCodecContext *s){
     ThreadContext *c= s->thread_opaque;
     int i;
 
@@ -77,7 +77,7 @@ void avcodec_pthread_free(AVCodecContext *s){
     av_freep(&s->thread_opaque);
 }
 
-int avcodec_pthread_execute(AVCodecContext *s, int (*func)(AVCodecContext *c2, void *arg2),void **arg, int *ret, int count){
+int avcodec_thread_execute(AVCodecContext *s, int (*func)(AVCodecContext *c2, void *arg2),void **arg, int *ret, int count){
     ThreadContext *c= s->thread_opaque;
     int i, val;
     
@@ -107,7 +107,7 @@ int avcodec_pthread_execute(AVCodecContext *s, int (*func)(AVCodecContext *c2, v
     return 0;
 }
 
-int avcodec_pthread_init(AVCodecContext *s, int thread_count){
+int avcodec_thread_init(AVCodecContext *s, int thread_count){
     int i;
     ThreadContext *c;
 
@@ -130,10 +130,10 @@ int avcodec_pthread_init(AVCodecContext *s, int thread_count){
     }
 //printf("init done\n"); fflush(stdout);
     
-    s->execute= avcodec_pthread_execute;
+    s->execute= avcodec_thread_execute;
 
     return 0;
 fail:
-    avcodec_pthread_free(s);
+    avcodec_thread_free(s);
     return -1;
 }
@@ -54,10 +54,10 @@ static void * thread_func(void *v){
 
 /**
- * free what has been allocated by avcodec_pthread_init().
- * must be called after decoding has finished, especially dont call while avcodec_pthread_execute() is running
+ * free what has been allocated by avcodec_thread_init().
+ * must be called after decoding has finished, especially dont call while avcodec_thread_execute() is running
  */
diff --git a/libavcodec/w32thread.c b/libavcodec/w32thread.c
new file mode 100644 (file)
index 0000000..186e302
--- /dev/null
@@ -0,0 +1,134 @@
+/*
+ * Copyright (c) 2004 Michael Niedermayer <michaelni@gmx.at>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+//#define DEBUG
+
+#include "avcodec.h"
+#include "common.h"
+
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <process.h>
+
+typedef struct ThreadContext{
+    AVCodecContext *avctx;
+    HANDLE thread;
+    HANDLE work_sem;
+    HANDLE done_sem;
+    int (*func)(AVCodecContext *c, void *arg);
+    void *arg;
+    int ret;
+}ThreadContext;
+
+
+static unsigned __stdcall thread_func(void *v){
+    ThreadContext *c= v;
+
+    for(;;){
+//printf("thread_func %X enter wait\n", (int)v); fflush(stdout);
+        WaitForSingleObject(c->work_sem, INFINITE);
+//printf("thread_func %X after wait (func=%X)\n", (int)v, (int)c->func); fflush(stdout);
+        if(c->func)
+            c->ret= c->func(c->avctx, c->arg);
+        else
+            return 0;
+//printf("thread_func %X signal complete\n", (int)v); fflush(stdout);
+        ReleaseSemaphore(c->done_sem, 1, 0);
+    }
+    
+    return 0;
+}
+
+/**
+ * free what has been allocated by avcodec_thread_init().
+ * must be called after decoding has finished, especially dont call while avcodec_thread_execute() is running
+ */
+void avcodec_thread_free(AVCodecContext *s){
+    ThreadContext *c= s->thread_opaque;
+    int i;
+
+    for(i=0; i<s->thread_count; i++){
+        
+        c[i].func= NULL;
+        ReleaseSemaphore(c[i].work_sem, 1, 0);
+        WaitForSingleObject(c[i].thread, INFINITE);
+        if(c[i].work_sem) CloseHandle(c[i].work_sem);
+        if(c[i].done_sem) CloseHandle(c[i].done_sem);
+    }
+
+    av_freep(&s->thread_opaque);
+}
+
+int avcodec_thread_execute(AVCodecContext *s, int (*func)(AVCodecContext *c2, void *arg2),void **arg, int *ret, int count){
+    ThreadContext *c= s->thread_opaque;
+    int i;
+    
+    assert(s == c->avctx);
+    assert(count <= s->thread_count);
+    
+    /* note, we can be certain that this is not called with the same AVCodecContext by different threads at the same time */
+
+    for(i=0; i<count; i++){
+        c[i].arg= arg[i];
+        c[i].func= func;
+        c[i].ret= 12345;
+
+        ReleaseSemaphore(c[i].work_sem, 1, 0);
+    }
+    for(i=0; i<count; i++){
+        WaitForSingleObject(c[i].done_sem, INFINITE);
+        
+        c[i].func= NULL;
+        if(ret) ret[i]= c[i].ret;
+    }
+    return 0;
+}
+
+int avcodec_thread_init(AVCodecContext *s, int thread_count){
+    int i;
+    ThreadContext *c;
+    uint32_t threadid;
+
+    s->thread_count= thread_count;
+
+    assert(!s->thread_opaque);
+    c= av_mallocz(sizeof(ThreadContext)*thread_count);
+    s->thread_opaque= c;
+    
+    for(i=0; i<thread_count; i++){
+//printf("init semaphors %d\n", i); fflush(stdout);
+        c[i].avctx= s;
+
+        if(!(c[i].work_sem = CreateSemaphore(NULL, 0, s->thread_count, NULL)))
+            goto fail;
+        if(!(c[i].done_sem = CreateSemaphore(NULL, 0, s->thread_count, NULL)))
+            goto fail;
+
+//printf("create thread %d\n", i); fflush(stdout);
+        c[i].thread = (HANDLE)_beginthreadex(NULL, 0, thread_func, &c[i], 0, &threadid );
+        if( !c[i].thread ) goto fail;
+    }
+//printf("init done\n"); fflush(stdout);
+    
+    s->execute= avcodec_thread_execute;
+
+    return 0;
+fail:
+    avcodec_thread_free(s);
+    return -1;
+}